# -*- coding: utf-8 -*-
"""
/***************************************************************************
 GEP_OnSSET
                                 A QGIS plugin
 Extracts GIS data to csv for OnSSET
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-11-18
        git sha              : $Format:%H$
        copyright            : (C) 2018 by Babak Khavari
        email                : babak.khavari@desa.kth.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the following packages in order to run code

from PyQt5.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt5.QtWidgets import *
import sys
import os
import shutil
from qgis.core import *
from qgis.gui import *
from PyQt5.QtGui import *
from processing.core.Processing import Processing
Processing.initialize()
import processing

# Initialize Qt resources from file resources.py
from .resources import *
# Import the code for the dialog
from .GEP_OnSSET_dialog import GEP_OnSSETDialog
import os.path


class GEP_OnSSET:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'GEP_OnSSET_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = GEP_OnSSETDialog()

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&GEP_OnSSET')
        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'GEP_OnSSET')
        self.toolbar.setObjectName(u'GEP_OnSSET')

        self.dlg.lineEdit.clear()
        self.dlg.toolButton.clicked.connect(self.select_workspace)

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('GEP_OnSSET', message)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToDatabaseMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/GEP_OnSSET/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'GEP_OnSSET'),
            callback=self.run,
            parent=self.iface.mainWindow())
        self.dlg.pushButton.clicked.connect(self.run)


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginDatabaseMenu(
                self.tr(u'&GEP_OnSSET'),
                action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar

    # Enetering the workspace directory in which the use want to save the outputs of the code. This directory needs to
    # exist
    def select_workspace(self):
        workspace = QFileDialog.getExistingDirectory(self.dlg, "Enter your workspace directory ", "")
        self.dlg.lineEdit.setText(workspace)

    # The name of the study area that is being used. The output file will have this name
    def study_area(self):
        settlement_fc = self.setObjectName(self.dlg, "Enter the name of your study area ", "")
        self.dlg.lineEdit_4.setText(settlement_fc)

    def projCord_func(self):
        projCord = self.mQgsProjectionSelectionWidget(self.dlg, "Enter the CRS")
        self.dlg.mQgsProjectionSelectionWidget.crs(projCord)


    def run(self):
        """Run method that performs all the real work"""
        layers = QgsProject.instance().mapLayers().values()
        for layer in layers:
            if layer.type() == QgsMapLayer.VectorLayer and (
                    layer.wkbType() == QgsWkbTypes.Polygon or layer.wkbType() == QgsWkbTypes.MultiPolygon):
                self.dlg.comboBox.addItem(layer.name(), layer)

            elif layer.type() == QgsMapLayer.VectorLayer and (
                    layer.wkbType() == QgsWkbTypes.Point or layer.wkbType() == QgsWkbTypes.MultiPoint):
                self.dlg.comboBox_12.addItem(layer.name(), layer)
                self.dlg.comboBox_19.addItem(layer.name(), layer)

            elif layer.type() == QgsMapLayer.VectorLayer and (
                    layer.wkbType() == QgsWkbTypes.LineString or layer.wkbType() == QgsWkbTypes.MultiLineString):
                self.dlg.comboBox_10.addItem(layer.name(), layer)
                self.dlg.comboBox_11.addItem(layer.name(), layer)
                self.dlg.comboBox_13.addItem(layer.name(), layer)
                self.dlg.comboBox_17.addItem(layer.name(), layer)
                self.dlg.comboBox_18.addItem(layer.name(), layer)
            else:
                self.dlg.comboBox_3.addItem(layer.name(), layer)
                self.dlg.comboBox_4.addItem(layer.name(), layer)
                self.dlg.comboBox_5.addItem(layer.name(), layer)
                self.dlg.comboBox_6.addItem(layer.name(), layer)
                self.dlg.comboBox_7.addItem(layer.name(), layer)
                self.dlg.comboBox_8.addItem(layer.name(), layer)

            self.dlg.comboBox_2.addItem(layer.name(), layer)
            self.dlg.comboBox_14.addItem(layer.name(), layer)


        layers = [layer for layer in QgsProject.instance().mapLayers().values()]
        layer_list = []
        for layer in layers:
            if layer.type()== QgsMapLayer.VectorLayer:
                layer_list.append(layer.name())

        def layer_field2():
            if self.dlg.comboBox_2.currentData().type() == QgsMapLayer.VectorLayer:
                selectedLayerIndex = self.dlg.comboBox_2.currentIndex()
                selectedLayer = layers[selectedLayerIndex]
                fields = selectedLayer.fields()
                fieldnames = [field.name() for field in fields]
                self.dlg.comboBox_20.clear()
                self.dlg.comboBox_20.addItems(fieldnames)

        self.dlg.comboBox_2.currentIndexChanged.connect(layer_field2)

        def layer_field():
            selectedLayerIndex = self.dlg.comboBox_14.currentIndex()
            selectedLayer = layers[selectedLayerIndex]
            fields = selectedLayer.fields()
            fieldnames = [field.name() for field in fields]
            self.dlg.comboBox_15.clear()
            self.dlg.comboBox_15.addItems(fieldnames)

        self.dlg.comboBox_14.currentIndexChanged.connect(layer_field)

        hydropowerFieldUnit = ["W", "kW", "MW"]
        self.dlg.comboBox_16.addItems(hydropowerFieldUnit)

        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result:
            # Do something useful here - delete the line containing pass and
            # substitute with your code.

            workspace = self.dlg.lineEdit.text()
            settlements_fc = self.dlg.lineEdit_4.text()
            projCord = self.dlg.mQgsProjectionSelectionWidget.crs().authid()
            hydropowerField = str(self.dlg.comboBox_15.currentText())
            popField = str(self.dlg.comboBox_20.currentText())
            hydropowerFieldUnit = str(self.dlg.comboBox_16.currentText())

            # We will create the folders needed for the analysis here. There is no need for the user to do so.
            # All the user has to do is to create the workspace mentioned above
            if not os.path.exists(workspace + r"/Assist"):
                os.makedirs(workspace + r"/Assist")

            if not os.path.exists(workspace + r"/Assist2"):
                os.makedirs(workspace + r"/Assist2")

            if not os.path.exists(workspace + r"/DEM"):
                os.makedirs(workspace + r"/DEM")

            if not os.path.exists(workspace + r"/Hydropower"):
                os.makedirs(workspace + r"/Hydropower")

            if not os.path.exists(workspace + r"/Land_Cover"):
                os.makedirs(workspace + r"/Land_Cover")

            if not os.path.exists(workspace + r"/Night_Time_Lights"):
                os.makedirs(workspace + r"/Night_Time_Lights")

            if not os.path.exists(workspace + r"/Population_2015"):
                os.makedirs(workspace + r"/Population_2015")

            if not os.path.exists(workspace + r"/Roads"):
                os.makedirs(workspace + r"/Roads")

            if not os.path.exists(workspace + r"/Slope"):
                os.makedirs(workspace + r"/Slope")

            if not os.path.exists(workspace + r"/Solar"):
                os.makedirs(workspace + r"/Solar")

            if not os.path.exists(workspace + r"/Transformers"):
                os.makedirs(workspace + r"/Transformers")

            if not os.path.exists(workspace + r"/Substations"):
                os.makedirs(workspace + r"/Substations")

            if not os.path.exists(workspace + r"/HV_Network"):
                os.makedirs(workspace + r"/HV_Network")

            if not os.path.exists(workspace + r"/MV_Network"):
                os.makedirs(workspace + r"/MV_Network")

            if not os.path.exists(workspace + r"/Travel_time"):
                os.makedirs(workspace + r"/Travel_time")

            if not os.path.exists(workspace + r"/Wind"):
                os.makedirs(workspace + r"/Wind")

            assistingFolder = workspace + r"/Assist"
            assistingFolder2 = workspace + r"/Assist2"

            # Defining all the datasets used in the analysis
            admin = self.dlg.comboBox.currentData()
            pop = self.dlg.comboBox_2.currentData()
            ghi = self.dlg.comboBox_3.currentData()
            windvel = self.dlg.comboBox_4.currentData()
            travel = self.dlg.comboBox_5.currentData()
            nightlight = self.dlg.comboBox_6.currentData()
            elevation = self.dlg.comboBox_7.currentData()
            land_cover = self.dlg.comboBox_8.currentData()
            existing_HV = self.dlg.comboBox_10.currentData()
            planned_HV = self.dlg.comboBox_11.currentData()
            substations = self.dlg.comboBox_12.currentData()
            roads = self.dlg.comboBox_13.currentData()
            hydro_points = self.dlg.comboBox_14.currentData()
            existing_MV = self.dlg.comboBox_17.currentData()
            planned_MV = self.dlg.comboBox_18.currentData()
            transformers = self.dlg.comboBox_19.currentData()
            slope = 'slope'

            # Creata a "extent" layer, this will be used to clip all the other datasets in the analysis, this way we
            # will not have null values caused by missing extents.
            ext = admin.extent()

            xmin = ext.xMinimum() - 1
            xmax = ext.xMaximum() + 1
            ymin = ext.yMinimum() - 1
            ymax = ext.yMaximum() + 1

            # Createas a coords string. This is important furher down below when we specify that the datasets should
            # be clipped with the extent
            coords = '{},{},{},{}'.format(xmin, xmax, ymin, ymax)

            # Population layer
            processing.run("native:reprojectlayer",
                           {'INPUT': pop, 'TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/Population_2015/pop2015.shp'})

            Pop = QgsVectorLayer(workspace + r'/Population_2015/pop2015.shp', '', 'ogr')

            # Renaming the population layer so that it can be used later
            for field in Pop.fields():
                if field.name() == popField:
                    with edit(Pop):
                        idx = Pop.fields().indexFromName(field.name())
                        Pop.renameAttribute(idx, 'pop2015' + settlements_fc[0:3])

            # Create virtual point layer for distances to be used later
            processing.run("qgis:selectbyexpression", {
                'INPUT': Pop, 'EXPRESSION': '\"Area\">=1', 'METHOD': 0})

            processing.run("native:saveselectedfeatures", {
                'INPUT': Pop,
                'OUTPUT': workspace + r'/Population_2015/pop2015_large.shp'})

            processing.run("qgis:pointsalonglines", {
                'INPUT': workspace + r'/Population_2015/pop2015_large.shp',
                'DISTANCE': 1000, 'START_OFFSET': 0, 'END_OFFSET': 0,
                'OUTPUT': assistingFolder+ r"/virtualpoints.shp"})

            settlement_points = QgsVectorLayer(assistingFolder + r'/virtualpoints.shp', '', 'ogr')

            # Creating centerpoints for clusters smaller than 1 km2
            processing.run("qgis:selectbyexpression", {
                'INPUT': Pop, 'EXPRESSION': '\"Area\"<1', 'METHOD': 0})

            processing.run("native:saveselectedfeatures", {
                'INPUT': Pop,
                'OUTPUT': workspace + r'/Population_2015/pop2015_small.shp'})

            processing.run("native:centroids", {
                'INPUT': workspace + r'/Population_2015/pop2015_small.shp',
                'ALL_PARTS': False, 'OUTPUT': assistingFolder  + r'/centerpoints.shp'})


            centerpoints = QgsVectorLayer(assistingFolder  + r'/centerpoints.shp', "", "ogr")

            # Rasters
            # Create elevation and slope maps.
            # 1. Clip the elevation map with the extent layer.
            # 2. Create a terrain slope map with the elevation layer
            # 3. Reprojects the slope and elevation maps to the coordinates specified above
            # 4. Interpolates the elevation and slope maps in order to avoid null values
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': elevation, 'PROJWIN': coords, 'NODATA': None,
                            'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/DEM/elevation_' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:slope",
                           {'INPUT': workspace + r'/DEM/elevation_' + settlements_fc[0:3] + '.tif', 'BAND': 1,
                            'SCALE': 111120, 'AS_PERCENT': False, 'COMPUTE_EDGES': False, 'ZEVENBERGEN': False,
                            'OPTIONS': '',
                            'OUTPUT': workspace + r"/Slope/" + slope + '_' + settlements_fc[0:3] + ".tif"})
            processing.run("gdal:fillnodata",
                           {'INPUT': workspace + r"/Slope/" + slope + '_' + settlements_fc[0:3] + ".tif", 'BAND': 1,
                            'DISTANCE': 50, 'ITERATIONS': 0, 'NO_MASK': False, 'MASK_LAYER': None,
                            'OUTPUT': assistingFolder2 + r'/' + slope + ".tif"})
            processing.run("gdal:fillnodata",
                           {'INPUT': workspace + r'/DEM/elevation_' + settlements_fc[0:3] + '.tif', 'BAND': 1,
                            'DISTANCE': 50, 'ITERATIONS': 0, 'NO_MASK': False, 'MASK_LAYER': None,
                            'OUTPUT': assistingFolder2 + r'/elevation.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': assistingFolder2 + r'/' + slope + ".tif", 'SOURCE_CRS': None,
                            'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0, 'OPTIONS': '', 'RESAMPLING': 0,
                            'DATA_TYPE': 5, 'TARGET_EXTENT': None, 'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Slope/' + slope + settlements_fc[0:3] + '_Proj.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': assistingFolder2 + r'/elevation.tif',
                            'SOURCE_CRS': None, 'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0,
                            'OPTIONS': '', 'RESAMPLING': 0, 'DATA_TYPE': 5, 'TARGET_EXTENT': None,
                            'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/DEM/elevation' + settlements_fc[0:3] + '_Proj.tif'})

            # GHI
            # 1. Clip the ghi map with the extent layer.
            # 2. Reprojects the ghi map to the coordinates specified above
            # 3. Interpolates the ghi map in order to avoid null values
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': ghi, 'PROJWIN': coords, 'NODATA': None,
                            'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/Solar/ghi_' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:fillnodata",
                           {'INPUT': workspace + r'/Solar/ghi_' + settlements_fc[0:3] + '.tif', 'BAND': 1,
                            'DISTANCE': 50, 'ITERATIONS': 0, 'NO_MASK': False, 'MASK_LAYER': None,
                            'OUTPUT': assistingFolder2 + r'/ghi.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': assistingFolder2 + r'/ghi.tif', 'SOURCE_CRS': None,
                            'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0, 'OPTIONS': '', 'RESAMPLING': 0,
                            'DATA_TYPE': 5, 'TARGET_EXTENT': None, 'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Solar/ghi_' + settlements_fc[0:3] + '_Proj.tif'})

            # Traveltime
            # 1. Clip the traveltime map with the extent layer.
            # 2. Reprojects the traveltime map to the coordinates specified above
            # 3. Interpolates the traveltime map in order to avoid null values
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': travel, 'PROJWIN': coords, 'NODATA': None,
                            'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/Travel_time/travel_' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:fillnodata",
                           {'INPUT': workspace + r'/Travel_time/travel_' + settlements_fc[0:3] + '.tif',
                            'BAND': 1, 'DISTANCE': 50, 'ITERATIONS': 0, 'NO_MASK': False, 'MASK_LAYER': None,
                            'OUTPUT': assistingFolder2 + r'/travel.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': assistingFolder2 + r'/travel.tif',
                            'SOURCE_CRS': None, 'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0,
                            'OPTIONS': '', 'RESAMPLING': 0, 'DATA_TYPE': 5, 'TARGET_EXTENT': None,
                            'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Travel_time/travel_' + settlements_fc[0:3] + '_Proj.tif'})

            # Wind
            # 1. Clip the wind velocity map with the extent layer.
            # 2. Reprojects the wind velocity map to the coordinates specified above
            # 3. Interpolates the wind velocity map in order to avoid null values
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': windvel, 'PROJWIN': coords, 'NODATA': None,
                            'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/Wind/windvel_' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:fillnodata",
                           {'INPUT': workspace + r'/Wind/windvel_' + settlements_fc[0:3] + '.tif', 'BAND': 1,
                            'DISTANCE': 50, 'ITERATIONS': 0, 'NO_MASK': False, 'MASK_LAYER': None,
                            'OUTPUT': assistingFolder2 + r'/windvel.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': assistingFolder2 + r'/windvel.tif', 'SOURCE_CRS': None,
                            'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0, 'OPTIONS': '', 'RESAMPLING': 0,
                            'DATA_TYPE': 5, 'TARGET_EXTENT': None, 'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Wind/windvel_' + settlements_fc[0:3] + '_Proj.tif'})

            # Landcover
            # 1. Clip the landcover map with the extent layer.
            # 2. Reprojects the landcover map to the coordinates specified above
            # This dataset is not interpolated as it is discrete
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': land_cover, 'PROJWIN': coords, 'NODATA': None, 'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/Land_Cover/land_cover_' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': workspace + r'/Land_Cover/land_cover_' + settlements_fc[0:3] + '.tif',
                            'SOURCE_CRS': None, 'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0,
                            'OPTIONS': '', 'RESAMPLING': 0, 'DATA_TYPE': 5, 'TARGET_EXTENT': None,
                            'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Land_Cover/land_cover_Proj.tif'})

            # Nighttimelights
            # 1. Clip the landcover map with the extent layer.
            # 2. Reprojects the landcover map to the coordinates specified above
            # This dataset is not interpolated as it is discrete
            processing.run("gdal:cliprasterbyextent",
                           {'INPUT': nightlight, 'PROJWIN': coords, 'NODATA': None, 'OPTIONS': '', 'DATA_TYPE': 5,
                            'OUTPUT': workspace + r'/Night_Time_Lights/nightlight' + settlements_fc[0:3] + '.tif'})
            processing.run("gdal:warpreproject",
                           {'INPUT': workspace + r'/Night_Time_Lights/nightlight' + settlements_fc[0:3] + '.tif',
                            'SOURCE_CRS': None, 'TARGET_CRS': projCord, 'NODATA': 0, 'TARGET_RESOLUTION': 0,
                            'OPTIONS': '', 'RESAMPLING': 0, 'DATA_TYPE': 5, 'TARGET_EXTENT': None,
                            'TARGET_EXTENT_CRS': None, 'MULTITHREADING': False,
                            'OUTPUT': workspace + r'/Night_Time_Lights/nightlight_Proj.tif'})

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"


            # Define all the rastermaps that have been generated this far
            traveltime = QgsRasterLayer(workspace + r'/Travel_time/travel_' + settlements_fc[0:3] + '_Proj.tif','traveltime')
            windvel = QgsRasterLayer(workspace + r'/Wind/windvel_' + settlements_fc[0:3] + '_Proj.tif', 'windvel')
            solar = QgsRasterLayer(workspace + r'/Solar/ghi_' + settlements_fc[0:3] + '_Proj.tif', 'solar')
            elevation = QgsRasterLayer(workspace + r'/DEM/elevation' + settlements_fc[0:3] + '_Proj.tif', 'elevation')
            slope = QgsRasterLayer(workspace + r'/Slope/' + slope + settlements_fc[0:3] + '_Proj.tif', 'slope')
            landcover = QgsRasterLayer(workspace + r'/Land_Cover/land_cover_Proj.tif', 'landcover')
            nightlights = QgsRasterLayer(workspace + r'/Night_Time_Lights/nightlight_Proj.tif', 'nightlights')

            # Add average wind speed
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': windvel, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [2]})

            # Add average ghi
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': solar, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [2]})

            # Add average traveltime
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': traveltime, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [2]})

            # Add average elevation
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': elevation, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [2]})

            # Add mode slope
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': slope, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [9]})

            # Add max ntl
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': nightlights, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [6]})

            # Add mode landcover
            processing.run("qgis:zonalstatistics", {'INPUT_RASTER': landcover, 'RASTER_BAND': 1, 'INPUT_VECTOR': Pop,
                                                    'COLUMN_PREFIX': '_', 'STATS': [9]})

            # Vector datasets
            # substations
            processing.run("native:clip",
                           {'INPUT': substations, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '.shp','FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/Substations/Substations_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/Substations/Substations_with_ID.shp','TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all intersecting polygon distance 0
            processing.run("native:selectbylocation",
                {'INPUT': Pop, 'PREDICATE': [0],'INTERSECT': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '_Proj.shp',
                'METHOD': 0})
            processing.run("native:saveselectedfeatures", {'INPUT': Pop,'OUTPUT': assistingFolder2 + r'/sub_int.shp'})
            processing.run("qgis:fieldcalculator",
                {'INPUT': assistingFolder2 + r'/sub_int.shp', 'FIELD_NAME': 'Substation', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': True, 'FORMULA': '0','OUTPUT': assistingFolder2 + r'/Substations_intersect.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': Pop, 'FIELD': 'ID','INPUT_2': assistingFolder2 + r'/Substations_intersect.shp','FIELD_2': 'ID',
                'FIELDS_TO_COPY': ['Substation'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Substationdist_intersecting.shp'})


            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                'HUBS': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '_Proj.shp','FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r"\Substationdist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\Substationdist.shp", 'VALUES_FIELD_NAME': 'HubDist', 'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/Substationdist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Substationdist_intersecting.shp', 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Substationdist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Substationdist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Substationdist_largerThanOne.shp',
                'FIELD_NAME': 'Substation', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"Substation\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/Substationdist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': centerpoints,
                'HUBS': workspace + r'/Substations/Substations' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3, 'OUTPUT': assistingFolder2 + r'/Substationdist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Substationdist_largerThanOne_andIntersect.shp', 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Substationdist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Substationdistplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Substationdistplaceholder.shp',
                'FIELD_NAME': 'Substation', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"Substation\" IS NULL, represent_value(\"HubDist\"), represent_value(\"Substation\"))',
                'OUTPUT': assistingFolder2 + r'/Substationdist.shp'})

            sub_dist = QgsVectorLayer(assistingFolder2 + r'/Substationdist.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': sub_dist,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/Substations/Substationdist.shp'})

            sub_dist = QgsVectorLayer(workspace + r'/Substations/Substationdist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # TRANSFORMERS
            processing.run("native:clip",
                           {'INPUT': transformers, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '.shp','FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/Transformers/Transformers_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/Transformers/Transformers_with_ID.shp','TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all intersecting polygon distance 0
            processing.run("native:selectbylocation",
                {'INPUT': Pop, 'PREDICATE': [0],'INTERSECT': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '_Proj.shp',
                'METHOD': 0})
            processing.run("native:saveselectedfeatures", {'INPUT': Pop,'OUTPUT': assistingFolder2 + r'/trans_int.shp'})
            processing.run("qgis:fieldcalculator",
                {'INPUT': assistingFolder2 + r'/trans_int.shp', 'FIELD_NAME': 'Transforme', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': True, 'FORMULA': '0','OUTPUT': assistingFolder2 + r'/Transformers_intersect.shp'})
            processing.run("native:joinattributestable", {
                'INPUT':sub_dist, 'FIELD': 'ID','INPUT_2': assistingFolder2 + r'/Transformers_intersect.shp','FIELD_2': 'ID',
                'FIELDS_TO_COPY': ['Transforme'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Transformerdist_intersecting.shp'})


            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                'HUBS': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '_Proj.shp','FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r"\Transformerdist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\Transformerdist.shp", 'VALUES_FIELD_NAME': 'HubDist', 'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/Transformerdist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Transformerdist_intersecting.shp', 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Transformerdist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Transformerdist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Transformerdist_largerThanOne.shp',
                'FIELD_NAME': 'Transforme', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"Transforme\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/Transformerdist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': centerpoints,
                'HUBS': workspace + r'/Transformers/Transformers' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3, 'OUTPUT': assistingFolder2 + r'/Transformerdist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Transformerdist_largerThanOne_andIntersect.shp', 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Transformerdist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Transformerdistplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Transformerdistplaceholder.shp',
                'FIELD_NAME': 'Transforme', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"Transforme\" IS NULL, represent_value(\"HubDist\"), represent_value(\"Transforme\"))',
                'OUTPUT': assistingFolder2 + r'/Transformerdist.shp'})

            trans_dist = QgsVectorLayer(assistingFolder2 + r'/Transformerdist.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': trans_dist,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/Transformers/Transformerdist.shp'})

            trans_dist = QgsVectorLayer(workspace + r'/Transformers/Transformerdist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"


            # Existing_HV_lines
            processing.run("native:clip",{'INPUT': existing_HV, 'OVERLAY': admin,
                'OUTPUT': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '.shp'})
            processing.run("saga:convertlinestopoints", {'LINES': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '.shp',
                'ADD         ': True, 'DIST': 0.000833333333,
                'POINTS': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + 'Point.shp'})
            processing.run("qgis:fieldcalculator",
                {'INPUT': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + 'Point.shp',
                'FIELD_NAME': 'AUTO','FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,
                'FORMULA': ' $id','OUTPUT': workspace + r'/HV_Network/Existing_HV_with_ID.shp'})
            processing.run("native:reprojectlayer",
                {'INPUT': workspace + r'/HV_Network/Existing_HV_with_ID.shp','TARGET_CRS': projCord,
                'OUTPUT': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                'HUBS': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '_Proj.shp','FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r"\EX_HV_dist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\EX_HV_dist.shp", 'VALUES_FIELD_NAME': 'HubDist', 'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/EX_HV_dist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': trans_dist, 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/EX_HV_dist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/EX_HV_dist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_HV_dist_largerThanOne.shp',
                'FIELD_NAME': 'EX_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"EX_HV\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/EX_HV_dist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': centerpoints,
                'HUBS': workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r'/EX_HV_Dist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/EX_HV_Dist_largerThanOne_andIntersect.shp',
                'FIELD': 'ID', 'INPUT_2': assistingFolder2 + r'/EX_HV_Dist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/EX_HV_Distplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_HV_Distplaceholder.shp',
                'FIELD_NAME': 'EX_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"EX_HV\" IS NULL, represent_value(\"HubDist\"), represent_value(\"EX_HV\"))',
                'OUTPUT': assistingFolder2 + r'/EX_HV_Dist_placeholder_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_HV_Dist_placeholder_2.shp',
                'FIELD_NAME': 'EX_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"EX_HV\" < 1, 0,\"EX_HV\")',
                'OUTPUT': assistingFolder2 + r'/EX_HV_dist_2.shp'})

            ex_hv = QgsVectorLayer(assistingFolder2 + r'/EX_HV_dist_2.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': ex_hv,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/HV_Network/Existing_HV_dist.shp'})

            ex_hv = QgsVectorLayer(workspace + r'/HV_Network/Existing_HV_dist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # Planend_HV_lines
            processing.run("native:clip",
                           {'INPUT': planned_HV, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '.shp'})
            processing.run("native:mergevectorlayers", {'LAYERS': [
                workspace + r'/HV_Network/Existing_HV' + settlements_fc[0:3] + '.shp',
                workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '.shp'],
                'CRS': None, 'OUTPUT':  workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '_merged.shp'})
            processing.run("saga:convertlinestopoints", {
                'LINES': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '_merged.shp',
                'ADD         ': True, 'DIST': 0.000833333333,
                'POINTS': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + 'Point.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + 'Point.shp',
                            'FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,
                            'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/HV_Network/Planned_HV_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/HV_Network/Planned_HV_with_ID.shp',
                            'TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                                                               'HUBS': workspace + r'/HV_Network/Planned_HV' + settlements_fc[
                                                                                                                0:3] + '_Proj.shp',
                                                               'FIELD': 'AUTO', 'UNIT': 3,
                                                               'OUTPUT': assistingFolder2 + r"\PL_HV_dist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\PL_HV_dist.shp", 'VALUES_FIELD_NAME': 'HubDist',
                'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/PL_HV_dist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': ex_hv, 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/PL_HV_dist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/PL_HV_dist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_HV_dist_largerThanOne.shp',
                'FIELD_NAME': 'PL_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"PL_HV\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/PL_HV_dist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': centerpoints,
                'HUBS': workspace + r'/HV_Network/Planned_HV' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r'/PL_HV_Dist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/PL_HV_Dist_largerThanOne_andIntersect.shp',
                'FIELD': 'ID', 'INPUT_2': assistingFolder2 + r'/PL_HV_Dist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/PL_HV_Distplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_HV_Distplaceholder.shp',
                'FIELD_NAME': 'PL_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"PL_HV\" IS NULL, represent_value(\"HubDist\"), represent_value(\"PL_HV\"))',
                'OUTPUT': assistingFolder2 + r'/PL_HV_Dist_placeholder_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_HV_Dist_placeholder_2.shp',
                'FIELD_NAME': 'PL_HV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"PL_HV\" < 1, 0,\"PL_HV\")',
                'OUTPUT': assistingFolder2 + r'/PL_HV_dist_2.shp'})

            pl_hv = QgsVectorLayer(assistingFolder2 + r'/PL_HV_dist_2.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': pl_hv,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/HV_Network/Planned_HV_dist.shp'})

            pl_hv = QgsVectorLayer(workspace + r'/HV_Network/Planned_HV_dist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # Existing_MV_lines
            processing.run("native:clip",
                           {'INPUT': existing_MV, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + '.shp'})
            processing.run("saga:convertlinestopoints", {
                'LINES': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + '.shp',
                'ADD         ': True, 'DIST': 0.000833333333,
                'POINTS': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + 'Point.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + 'Point.shp',
                            'FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,
                            'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/MV_Network/Existing_MV_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/MV_Network/Existing_MV_with_ID.shp',
                            'TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                                                               'HUBS': workspace + r'/MV_Network/Existing_MV' + settlements_fc[
                                                                                                                0:3] + '_Proj.shp',
                                                               'FIELD': 'AUTO', 'UNIT': 3,
                                                               'OUTPUT': assistingFolder2 + r"\EX_MV_dist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\EX_MV_dist.shp", 'VALUES_FIELD_NAME': 'HubDist',
                'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/EX_MV_dist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': pl_hv, 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/EX_MV_dist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/EX_MV_dist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_MV_dist_largerThanOne.shp',
                'FIELD_NAME': 'EX_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"EX_MV\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/EX_MV_dist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': centerpoints,
                'HUBS': workspace + r'/MV_Network/Existing_MV' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r'/EX_MV_Dist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/EX_MV_Dist_largerThanOne_andIntersect.shp',
                'FIELD': 'ID', 'INPUT_2': assistingFolder2 + r'/EX_MV_Dist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/EX_MV_Distplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_MV_Distplaceholder.shp',
                'FIELD_NAME': 'EX_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"EX_MV\" IS NULL, represent_value(\"HubDist\"), represent_value(\"EX_MV\"))',
                'OUTPUT': assistingFolder2 + r'/EX_MV_Dist_placeholder_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/EX_MV_Dist_placeholder_2.shp',
                'FIELD_NAME': 'EX_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"EX_MV\" < 1, 0,\"EX_MV\")',
                'OUTPUT': assistingFolder2 + r'/EX_MV_dist_2.shp'})

            ex_mv = QgsVectorLayer(assistingFolder2 + r'/EX_MV_dist_2.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': ex_mv,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/MV_Network/Existing_MV_dist.shp'})

            ex_mv = QgsVectorLayer(workspace + r'/MV_Network/Existing_MV_dist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # Planned_MV_lines
            processing.run("native:clip",
                           {'INPUT': planned_MV, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + '.shp'})
            processing.run("saga:convertlinestopoints", {
                'LINES': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + '.shp',
                'ADD         ': True, 'DIST': 0.000833333333,
                'POINTS': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + 'Point.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + 'Point.shp',
                            'FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,
                            'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/MV_Network/Planned_MV_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/MV_Network/Planned_MV_with_ID.shp',
                            'TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                                                               'HUBS': workspace + r'/MV_Network/Planned_MV' + settlements_fc[
                                                                                                                0:3] + '_Proj.shp',
                                                               'FIELD': 'AUTO', 'UNIT': 3,
                                                               'OUTPUT': assistingFolder2 + r"\PL_MV_dist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\PL_MV_dist.shp", 'VALUES_FIELD_NAME': 'HubDist',
                'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/PL_MV_dist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': ex_mv, 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/PL_MV_dist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/PL_MV_dist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_MV_dist_largerThanOne.shp',
                'FIELD_NAME': 'PL_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"PL_MV\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/PL_MV_dist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': centerpoints,
                'HUBS': workspace + r'/MV_Network/Planned_MV' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r'/PL_MV_Dist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/PL_MV_Dist_largerThanOne_andIntersect.shp',
                'FIELD': 'ID', 'INPUT_2': assistingFolder2 + r'/PL_MV_Dist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/PL_MV_Distplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_MV_Distplaceholder.shp',
                'FIELD_NAME': 'PL_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"PL_MV\" IS NULL, represent_value(\"HubDist\"), represent_value(\"PL_MV\"))',
                'OUTPUT': assistingFolder2 + r'/PL_MV_Dist_placeholder_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/PL_MV_Dist_placeholder_2.shp',
                'FIELD_NAME': 'PL_MV', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"PL_MV\" < 1, 0,\"PL_MV\")',
                'OUTPUT': assistingFolder2 + r'/PL_MV_dist_2.shp'})

            pl_mv = QgsVectorLayer(assistingFolder2 + r'/PL_MV_dist_2.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': pl_mv,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/MV_Network/Planned_MV_dist.shp'})

            pl_mv = QgsVectorLayer(workspace + r'/MV_Network/Planned_MV_dist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # Roads
            processing.run("native:clip",
                           {'INPUT': roads, 'OVERLAY': admin,
                            'OUTPUT': workspace + r'/Roads/Roads' + settlements_fc[0:3] + '.shp'})
            processing.run("saga:convertlinestopoints", {
                'LINES': workspace + r'/Roads/Roads' + settlements_fc[0:3] + '.shp',
                'ADD         ': True, 'DIST': 0.000833333333,
                'POINTS': workspace + r'/Roads/Roads' + settlements_fc[0:3] + 'Point.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': workspace + r'/Roads/Roads' + settlements_fc[0:3] + 'Point.shp',
                            'FIELD_NAME': 'AUTO',
                            'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,
                            'FORMULA': ' $id',
                            'OUTPUT': workspace + r'/Roads/Roads_with_ID.shp'})
            processing.run("native:reprojectlayer",
                           {'INPUT': workspace + r'/Roads/Roads_with_ID.shp',
                            'TARGET_CRS': projCord,
                            'OUTPUT': workspace + r'/Roads/Roads' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {'INPUT': settlement_points,
                                                               'HUBS': workspace + r'/Roads/Roads' + settlements_fc[
                                                                                                                0:3] + '_Proj.shp',
                                                               'FIELD': 'AUTO', 'UNIT': 3,
                                                               'OUTPUT': assistingFolder2 + r"\Roads_dist.shp"})
            processing.run("qgis:statisticsbycategories", {
                'INPUT': assistingFolder2 + r"\Roads_dist.shp", 'VALUES_FIELD_NAME': 'HubDist',
                'CATEGORIES_FIELD_NAME': ['ID'],
                'OUTPUT': assistingFolder2 + r'/Roads_dist_largerThanOne.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': pl_mv, 'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Roads_dist_largerThanOne.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Roads_dist_largerThanOne.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Roads_dist_largerThanOne.shp',
                'FIELD_NAME': 'Roads', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"Roads\" IS NULL, represent_value(\"min\"),0)',
                'OUTPUT': assistingFolder2 + r'/Roads_dist_largerThanOne_andIntersect.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': centerpoints,
                'HUBS': workspace + r'/Roads/Roads' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3,
                'OUTPUT': assistingFolder2 + r'/Roads_Dist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Roads_Dist_largerThanOne_andIntersect.shp',
                'FIELD': 'ID', 'INPUT_2': assistingFolder2 + r'/Roads_Dist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Roads_Distplaceholder.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Roads_Distplaceholder.shp',
                'FIELD_NAME': 'Roads', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"Roads\" IS NULL, represent_value(\"HubDist\"), represent_value(\"Roads\"))',
                'OUTPUT': assistingFolder2 + r'/Roads_Dist_placeholder_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Roads_Dist_placeholder_2.shp',
                'FIELD_NAME': 'Roads', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"Roads\" < 1, 0,\"Roads\")',
                'OUTPUT': assistingFolder2 + r'/Roads_dist_2.shp'})

            roads = QgsVectorLayer(assistingFolder2 + r'/Roads_dist_2.shp', '', 'ogr')

            processing.run("qgis:deletecolumn", {
                'INPUT': roads,
                'COLUMN': ['min', 'HubDist'],
                'OUTPUT': workspace + r'/Roads/Roads_dist.shp'})

            roads = QgsVectorLayer(workspace + r'/Roads/Roads_dist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            # Hydro
            processing.run("native:clip",
                {'INPUT': hydro_points, 'OVERLAY': admin,
                'OUTPUT': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '.shp'})
            processing.run("qgis:fieldcalculator",
                {'INPUT': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '.shp','FIELD_NAME': 'AUTO',
                'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': True,'FORMULA': ' $id',
                'OUTPUT': workspace + r'/Hydropower/Hydro_with_ID.shp'})
            processing.run("native:reprojectlayer",
                {'INPUT': workspace + r'/Hydropower/Hydro_with_ID.shp','TARGET_CRS': projCord,
                'OUTPUT': assistingFolder2 + r'/Hydro' + settlements_fc[0:3] + '_Proj.shp'})
            processing.run("native:multiparttosingleparts", {
                'INPUT': assistingFolder2 + r'/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                'OUTPUT': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp'})

            # Give all intersecting polygon distance 0
            processing.run("native:selectbylocation",
                {'INPUT': Pop, 'PREDICATE': [0],
                'INTERSECT': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp','METHOD': 0})
            processing.run("native:saveselectedfeatures",{'INPUT': Pop, 'OUTPUT': assistingFolder2 + r'/Hydro_int.shp'})
            processing.run("qgis:fieldcalculator",
                           {'INPUT': assistingFolder2 + r'/Hydro_int.shp',
                            'FIELD_NAME': 'HydroDist', 'FIELD_TYPE': 2, 'FIELD_LENGTH': 255, 'FIELD_PRECISION': 3,
                            'NEW_FIELD': True, 'FORMULA': '0', 'OUTPUT': assistingFolder2 + r'/Hydro_intersect1.shp'})
            processing.run("native:joinattributestable",
                           {'INPUT': roads,'FIELD': 'ID','INPUT_2': assistingFolder2 + r'/Hydro_intersect1.shp',
                            'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HydroDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False,
                            'PREFIX': '', 'OUTPUT': assistingFolder2 + r'/Hydro_dist_intersecting1.shp'})

            processing.run("saga:pointstatisticsforpolygons", {
                'POINTS': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                'POLYGONS': assistingFolder2 + r'/Hydro_dist_intersecting1.shp',
                'FIELDS': hydropowerField, 'FIELD_NAME': 3, 'SUM             ': True, 'AVG             ': False,
                'VAR             ': False, 'DEV             ': False, 'MIN             ': False,
                'MAX             ': False, 'NUM             ': False,
                'STATISTICS': assistingFolder2 + r'/Hydro_intersect2.shp'})
            processing.run("saga:pointstatisticsforpolygons", {
                'POINTS': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                'POLYGONS': assistingFolder2 + r'/Hydro_intersect2.shp',
                'FIELDS': 'AUTO', 'FIELD_NAME': 3, 'SUM             ': False, 'AVG             ': False,
                'VAR             ': False, 'DEV             ': False, 'MIN             ': False,
                'MAX             ': True, 'NUM             ': False,
                'STATISTICS': assistingFolder2 + r'/Hydro_intersect3.shp'})

            hydro_dist = QgsVectorLayer(assistingFolder2 + r'/Hydro_intersect3.shp', "", "ogr")

            for field in hydro_dist.fields():
                if field.name() == 'SUM':
                    with edit(hydro_dist):
                        idx = hydro_dist.fields().indexFromName(field.name())
                        hydro_dist.renameAttribute(idx, 'Hydropower')

            for field in hydro_dist.fields():
                if field.name() == 'MAX':
                    with edit(hydro_dist):
                        idx = hydro_dist.fields().indexFromName(field.name())
                        hydro_dist.renameAttribute(idx, 'HydroFID')

            # Give all polygons larger than 1 km2 the right distance and merge with intersecting
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': settlement_points,
                'HUBS': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': 'AUTO', 'UNIT': 3, 'OUTPUT': assistingFolder2 + r"\Hydrodist_1.shp"})
            processing.run("qgis:statisticsbycategories",
                           {'INPUT': assistingFolder2 + r"\Hydrodist_1.shp",'VALUES_FIELD_NAME': 'HubDist',
                            'CATEGORIES_FIELD_NAME': ['ID', 'HubName'],
                            'OUTPUT': assistingFolder2 + r'/Hydrodist_largerThanOne1.csv'})
            processing.run("native:joinattributestable", {
                'INPUT': hydro_dist,'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Hydrodist_largerThanOne1.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['min'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT': assistingFolder2 + r'/Hydrodist_largerThanOne1.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Hydrodist_largerThanOne1.shp',
                'FIELD_NAME': 'HydroDist', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False, 'FORMULA': 'if(\"HydroDist\" IS NULL,represent_value(\"min\"),0)',
                'OUTPUT':  assistingFolder2 + r'/Hydrodist_largerThanOne2.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Hydrodist_largerThanOne2.shp','FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Hydrodist_largerThanOne1.csv',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubName'], 'METHOD': 1, 'DISCARD_NONMATCHING': False,
                'PREFIX': '', 'OUTPUT': assistingFolder2 + r"\Hydrodist_2.shp"})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r"\Hydrodist_2.shp",
                'FIELD_NAME': 'HydroFID', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                'FORMULA': 'if(\"HydroFID\" IS NULL, represent_value(\"HubName\"), \"HydroFID\")', 'OUTPUT': assistingFolder2 + r"\Hydrodist_3.shp"})
            processing.run("qgis:distancetonearesthubpoints", {
                'INPUT': settlement_points,
                'HUBS': workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                'FIELD': hydropowerField, 'UNIT': 0, 'OUTPUT': assistingFolder2 + r"\Hydrodist_4.shp"})
            processing.run("native:joinattributestable", {
                'INPUT':  assistingFolder2 + r"\Hydrodist_3.shp",'FIELD': 'ID','INPUT_2': assistingFolder2 + r"\Hydrodist_4.shp",
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubName'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT':  assistingFolder2 + r"\Hydrodist_5.shp"})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r"\Hydrodist_5.shp",
                'FIELD_NAME': 'Hydropower', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                'FORMULA': 'if(\"Hydropower\" IS NULL, represent_value(\"HubName_2\"),\"Hydropower\")', 'OUTPUT': assistingFolder2 + r"\Hydrodist_6.shp"})
            processing.run("qgis:deletecolumn", {
                'INPUT': assistingFolder2 + r"\Hydrodist_6.shp",
                'COLUMN': ['min','HubName', 'HubName_2'], 'OUTPUT': workspace + r'/Hydropower/Hydro_intersect_largerThan10.shp'})

            # Give all polygons smaller than 1 the right distance value and merge with intersecting + larger than 1.
            processing.run("qgis:distancetonearesthubpoints",
                           {'INPUT': centerpoints,'HUBS':  workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                            'FIELD': 'AUTO', 'UNIT': 3, 'OUTPUT':  assistingFolder2 + r'/Hydrodist_smallerThanOne.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': workspace + r'/Hydropower/Hydro_intersect_largerThan10.shp',
                'FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Hydrodist_smallerThanOne.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubName', 'HubDist'], 'METHOD': 1, 'DISCARD_NONMATCHING': False,
                'PREFIX': '', 'OUTPUT':  assistingFolder2 + r'/Hydrodist_smallerThanOne_2.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_2.shp',
                'FIELD_NAME': 'HydroDist', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3,
                'NEW_FIELD': False,
                'FORMULA': 'if(\"HydroDist\" is NULL, represent_value(\"HubDist\"), represent_value(\"HydroDist\"))',
                'OUTPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_3.shp'})
            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_3.shp',
                'FIELD_NAME': 'HydroFID', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                'FORMULA': 'if(\"HydroFID\" IS NULL, represent_value(\"HubName\"), represent_value(\"HydroFID\"))',
                'OUTPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_4.shp'})

            processing.run("qgis:distancetonearesthubpoints",
                           {'INPUT': centerpoints,
                            'HUBS':  workspace + r'/Hydropower/Hydro' + settlements_fc[0:3] + '_Proj.shp',
                            'FIELD': hydropowerField, 'UNIT': 3, 'OUTPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_5.shp'})
            processing.run("native:joinattributestable", {
                'INPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_4.shp','FIELD': 'ID',
                'INPUT_2': assistingFolder2 + r'/Hydrodist_smallerThanOne_5.shp',
                'FIELD_2': 'ID', 'FIELDS_TO_COPY': ['HubName'], 'METHOD': 1, 'DISCARD_NONMATCHING': False, 'PREFIX': '',
                'OUTPUT':assistingFolder2 + r'/Hydrodist_smallerThanOne_6.shp'})

            processing.run("qgis:fieldcalculator", {
                'INPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_6.shp',
                'FIELD_NAME': 'Hydropower', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                'FORMULA': 'if(\"Hydropower\" IS NULL, represent_value(\"HubName_2\"),represent_value(\"Hydropower\"))',
                'OUTPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_7.shp'})

            processing.run("qgis:deletecolumn", {
                'INPUT': assistingFolder2 + r'/Hydrodist_smallerThanOne_7.shp',
                'COLUMN': ['HubName', 'HubDist', 'HubName_2'], 'OUTPUT':workspace + r'/Hydropower/Hydrodist.shp'})

            hydro_dist = QgsVectorLayer(workspace + r'/Hydropower/Hydrodist.shp', '', 'ogr')

            shutil.rmtree(workspace + r"/Assist2")
            os.makedirs(workspace + r"/Assist2")
            assistingFolder2 = workspace + r"/Assist2"

            if hydropowerFieldUnit == "W":
                processing.run("qgis:fieldcalculator",
                               {'INPUT': hydro_dist, 'FIELD_NAME': 'Hydropower',
                                'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                                'FORMULA': ' \"Hydropower\" /1000',
                                'OUTPUT': assistingFolder + r'/' + settlements_fc + '.shp'})
            elif hydropowerFieldUnit == "MW":
                processing.run("qgis:fieldcalculator",
                               {'INPUT': hydro_dist, 'FIELD_NAME': 'Hydropower',
                                'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                                'FORMULA': ' \"Hydropower\" *1000',
                                'OUTPUT': assistingFolder + r'/' + settlements_fc + '.shp'})
            else:
                processing.run("qgis:fieldcalculator",
                               {'INPUT': hydro_dist, 'FIELD_NAME': 'Hydropower',
                                'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                                'FORMULA': ' \"Hydropower\" *1',
                                'OUTPUT': assistingFolder + r'/' + settlements_fc + '.shp'})



            settlements = QgsVectorLayer(assistingFolder + r'/' + settlements_fc + '.shp', "", "ogr")

            iter1 = processing.run("qgis:fieldcalculator", {'INPUT':assistingFolder + r'/' + settlements_fc + '.shp','FIELD_NAME':
                                    '_max','FIELD_TYPE':0,'FIELD_LENGTH':10,'FIELD_PRECISION':3,'NEW_FIELD':False,
                                    'FORMULA':'CASE WHEN  \"_max\" IS NULL THEN 0 ELSE \"_max\" END',
                                    'OUTPUT':assistingFolder2 + r"\iter1.shp"})
            iter2 = processing.run("qgis:fieldcalculator", {'INPUT':assistingFolder2 + r"\iter1.shp",'FIELD_NAME':
                                    '_majorit_1','FIELD_TYPE':0,'FIELD_LENGTH':10,'FIELD_PRECISION':3,'NEW_FIELD':False,
                                    'FORMULA':'CASE WHEN  \"_majorit_1\" IS NULL THEN 0 ELSE \"_majorit_1\" END',
                                    'OUTPUT':assistingFolder2 + r"\iter2.shp"})
            iter3 = processing.run("qgis:fieldcalculator", {'INPUT': assistingFolder2 + r"\iter2.shp", 'FIELD_NAME':
                                    '_majority', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                                    'FORMULA': 'CASE WHEN  \"_majority\" IS NULL THEN 0 ELSE \"_majority\" END',
                                    'OUTPUT': assistingFolder2 + r"\iter3.shp"})
            iter4 = processing.run("qgis:fieldcalculator", {'INPUT': assistingFolder2 + r"\iter3.shp", 'FIELD_NAME':
                                    '_mean_2', 'FIELD_TYPE': 0, 'FIELD_LENGTH': 10, 'FIELD_PRECISION': 3, 'NEW_FIELD': False,
                                    'FORMULA': 'CASE WHEN  \"_mean_2\" IS NULL THEN 0 ELSE \"_mean_2\" /60  END',
                                    'OUTPUT': assistingFolder2 + r"\iter4.shp"})

            processing.run("qgis:refactorfields",
                           {'INPUT': assistingFolder2 + r"\iter4.shp",
                            'FIELDS_MAPPING': [
                                {'expression': '"pop2015'+settlements_fc[0:3]+'"', 'length': 18, 'name': 'Pop', 'precision': 10,
                                 'type': 6},
                                {'expression': '"Area"', 'length': 18, 'name': 'GridCellArea', 'precision': 10, 'type': 6},
                                {'expression': '"ID"', 'length': 9, 'name': 'ID', 'precision': 0, 'type': 10},
                                {'expression': '"NTLArea"', 'length': 18, 'name': 'NTLArea', 'precision': 10,
                                 'type': 6},
                                {'expression': '"NTLBin"', 'length': 18, 'name': 'NTLBin', 'precision': 10, 'type': 6},
                                {'expression': '"NTLPop"', 'length': 18, 'name': 'ElecPop', 'precision': 10, 'type': 6},
                                {'expression': '"_mean"', 'length': 18, 'name': 'WindVel', 'precision': 10, 'type': 6},
                                {'expression': '"_mean_1"', 'length': 18, 'name': 'GHI', 'precision': 10,
                                 'type': 6},
                                {'expression': '"_mean_2"', 'length': 18, 'name': 'TravelHours', 'precision': 10,
                                 'type': 6},
                                {'expression': '"_mean_3"', 'length': 18, 'name': 'Elevation', 'precision': 10,
                                 'type': 6},
                                {'expression': '"_majority"', 'length': 18, 'name': 'Slope', 'precision': 10,
                                 'type': 6},
                                {'expression': '"_max"', 'length': 18, 'name': 'NightLights', 'precision': 10, 'type': 6},
                                {'expression': '"_majorit_1"', 'length': 18, 'name': 'LandCover', 'precision': 10,
                                 'type': 6},
                                {'expression': '"Substation"', 'length': 18, 'name': 'SubstationDist', 'precision': 10,
                                 'type': 6},
                                {'expression': '"Transforme"', 'length': 18, 'name': 'TransformerDist', 'precision': 10,
                                 'type': 6},
                                {'expression': '"EX_HV"', 'length': 18, 'name': 'CurrentHVLineDist', 'precision': 10, 'type': 6},
                                {'expression': '"PL_HV"', 'length': 18, 'name': 'PlannedHVLineDist', 'precision': 10, 'type': 6},
                                {'expression': '"EX_MV"', 'length': 18, 'name': 'CurrentMVLineDist', 'precision': 10, 'type': 6},
                                {'expression': '"PL_MV"', 'length': 18, 'name': 'PlannedMVLineDist', 'precision': 10, 'type': 6},
                                {'expression': '"Roads"', 'length': 18, 'name': 'RoadDist', 'precision': 10, 'type': 6},
                                {'expression': '"HydroDist"', 'length': 18, 'name': 'HydropowerDist', 'precision': 0,
                                 'type': 10},
                                {'expression': '"Hydropower"', 'length': 18, 'name': 'Hydropower', 'precision': 10,
                                 'type': 6},
                                {'expression': '"HydroFID"', 'length': 18, 'name': 'HydropowerFID', 'precision': 10,
                                 'type': 6}], 'OUTPUT':assistingFolder + r'/' + settlements_fc + '.csv'})

